#!/bin/sh
# \
exec tclsh $0 $@
package require Expect

# Copyright 2005 Jeff Jahr <malakais@pacbell.net> Sun Apr 24 13:42:02 PDT 2005

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.

# rokusbchime.tcl
#   This program connects to a Roku SoundBridge device on the open cli
#   interface port.  It puts the rokusb into 'sketch mode', and draws some
#   stuff.  The intention was that it be called from the crontab every fifteen
#   minutes to show the time and a witty aphorism from fortune, but feel free
#   to modify it to show whatever pertinant information suits you.  The whole
#   of this program took about two hours to write, so do whatever you want with
#   it. :)  By the way... if the program fails, it can leave the roku in sketch
#   mode, which tends to lock up the display.  If that happens, you can telnet
#   directly to the roku and type 'sketch' and 'quit'.

# Progam starts with [main]. See last line.


# proc rokusb_open:
#   Opens a connection to the RokuSB, returns a spawnid.
#
# Arguments:
#   rokusbip- Hostname or ip address of the RokuSB
#   port    - An optional port number, in case it ever changes.
#
# Results:
#   Success - expect spawnid for the connection.
#   Error   - not handled.
#
proc rokusb_open { rokusbip { port 4444 }} { 
	global stdprompt
	set rchan [socket $rokusbip $port]
	set expect_timeout 30
	log_user 0
	spawn -open $rchan
	expect $stdprompt
	return $spawn_id
}

# proc rokusb_close:
#   Tries to cleanly close down an existing connection to the RokuSB
#
# Arguments:
#   rspawn  - spawnid for the connection
#
# Results:
#   Success - Closes the connection, returns ""
#   Error   - not handled.
#
proc rokusb_close { rspawn } {
	set spawn_id $rspawn
	send "exit\r\n"
	close $rspawn
	return ""
}

# proc rokusb_opensketch:
#   Puts the RokuSB into sketch mode.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#
# Results:
#   Success - Puts roku into sketch mode, returns ""
#   Error   - Not handled.
#
proc rokusb_opensketch { rspawn } {
	set spawn_id $rspawn
	# assume were at a SoundBridge prompt, ready to go.
	send "sketch\r\n"
	expect "sketch>"
	return ""
}

# proc rokusb_closesketch:
#   Takes RokuSB out of sketch mode.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#
# Results:
#   Success - quits sketchmode, returns ""
#   Error   - not handled.
#
proc rokusb_closesketch { rspawn } {

	global stdprompt
	set spawn_id $rspawn
	send "quit\r\n"
	expect $stdprompt
	return ""
}

# proc rokusb_marquee:
#   A slightly fancier interface into the sketch marquee command.  It can take
#   a multiline message, escape the quotes, and call marquee once for each line
#   in msg.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#   msg     - The message to be scrolled across the screen.
#   font    - The font to use.  defaults to 2, the big one.
#
# Results:
#   Success - scrolls the message, returns ""
#   Error   - not handled.
#
proc rokusb_marquee { rspawn msg { font 2 } } {

	set spawn_id $rspawn

	rokusb_font $rspawn $font 
	set msg [string map { \" \\\" } $msg]

	foreach line [split $msg "\n"] {
		set cmd "marquee \"$line\"\r\n"
		send $cmd
		expect "sketch>"
	}
	return ""
}

# proc rokusb_text:
#   Interface into the sketch text command with reasonable defaults.  Escapes
#   quotemarks, and will only print out the first line of the message.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#   msg     - The message to be scrolled across the screen.
#   font    - The font to use.  defaults to 2, the big one.
#   x       - Where on the screen to draw, in pixels.
#   y       - Where on the screen to draw, in pixels.
#
# Results:
#   Success - drawns the text, returns ""
#   Error   - Not handled.
#
proc rokusb_text { rspawn msg { font 2 } { x 0 } { y 0 }} {

	set spawn_id $rspawn

	rokusb_font $rspawn $font

	set msg [string map { \" \\\" } $msg]
	set line [lindex [split $msg "\n"] 0]
	set cmd "text $x $y \"$line\"\r\n"
	send $cmd
	expect "sketch>"
	return ""
}

# proc rokusb_font:
#   wrapper around the sketch font command.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#
# Results:
#   Success - sets the font.
#   Error   - not handled.
#
proc rokusb_font { rspawn font } {
	set spawn_id $rspawn
	send "font $font\r\n"
	expect "sketch>"
	return ""
}

# proc rokusb_clear:
#   wrapper around the sketch clear command.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#
# Results:
#   Success - clears the screen.
#   Error   - not handled.
#
proc rokusb_clear { rspawn } {
	set spawn_id $rspawn
	send "clear\r\n"
	expect "sketch>"
	return ""
}

# proc rokusb_clock:
#   Displays the time of day in this format "Sun Apr 24 01:25 PM".  Optionally
#   clears the screen after a given number of milliseconds.
#
# Arguments:
#   rspawn  - spawnid for the connection.
#   clear   - if non zero, wait this many ms and clear the screen.
#
# Results:
#   Success - Puts the time on the screen.
#   Error   - not handled.
#
proc rokusb_clock { rspawn { clear 0 } } {

	set date [exec date {+%a %b %e %I:%M %p}]
	rokusb_text $rspawn $date

	if { $clear != 0 } {
		after $clear
		rokusb_clear $rspawn
	}

}

# proc fortuneclock:
#   A fancy display hack.  Shows the time, scrolls a short aphorism from the
#   fortune command, shows the time, and exits.  This can be used as an example
#   for how to display an occasional message on the RokuSB.
#
# Arguments:
#   rspawn  - What the argument is for.
#
# Results:
#   Success - All that.
#   Error   - sue me, im lazy.
#
proc fortuneclock { rspawn } {

	# gotta be in sketch mode first.
	rokusb_opensketch $rspawn

	rokusb_clock $rspawn 5000

	if { [catch {exec fortune -a -s} msg] } {
		puts "$msg"
	}
	rokusb_marquee $rspawn $msg

	rokusb_clock $rspawn 5000

	# give the dispaly back to the user.
	rokusb_closesketch $rspawn
}

proc main { argv } {
	# first argument is the ip address of the RokuSB.
	set rokusbip [lindex $argv 0]
	if { $rokusbip == "" || $rokusbip == "-h" } {
		puts "Usage: [info script] <ip address or hostname of RokuSB>"
		exit
	}

	if { [catch {rokusb_open $rokusbip} rspawn] } {
		puts "$rspawn"
		exit
	}

	fortuneclock $rspawn
	rokusb_close $rspawn
}

# In case the default prompt changes.  Heh.
set stdprompt "SoundBridge>"

main $argv

# Hell, this thing is more comments than code!  ;') -jsj
